@@ -65,14 +65,14 @@ setMethod("initialize",
 		callNextMethod(.Object, states = states, byrow = byrow, transitionMatrix=transitionMatrix,name=name,...)
     }
 )
-		 
+
 # #test
 # stateNames=c("a","b")
 # ciao<-new("markovchain", states=stateNames, transitionMatrix=matrix(c(1,0,0,1),byrow=TRUE, nrow=2, 
 #                                                  dimnames=list(stateNames,stateNames)
 #                                                  ))
 
-#@TAE: try to see if it can be moved to Rcpp efficiently
+
 
 # .isProb<-function(prob)
 # {
@@ -92,6 +92,14 @@ setMethod("states","markovchain",
           }
 )
 
+#adding a method names
+setMethod("names","markovchain", 
+          function(x) {
+            out <- x@states
+            return(out)
+          }
+)
+
 
 
  # generic methods to get the dim of a markovchain and markovchainList
@@ -130,8 +138,6 @@ setValidity("markovchain",
 		}
 )
 
-
-
 .mcEigen<-function(matr, transpose=TRUE)
 {
  # Function to extract eigenvalues, core of get steady states 
@@ -285,9 +291,13 @@ setMethod("print","markovchain", #metodo print
 	#
 	# a graph adjacenty
 	if (object@byrow==FALSE) object <- t(object)
-	matr <- Matrix(data=object@transitionMatrix, sparse=TRUE)*100 #need to multiply by 100
+	#with this fails 
+	#matr <- Matrix(data=object@transitionMatrix, sparse = TRUE)*100 #need to multiply by 100
+	#with this works. 
+	matr<-object@transitionMatrix*100
 	if(round==TRUE) matr <- round(matr,2)
 	net <- graph.adjacency(adjmatrix=matr, weighted=TRUE, mode="directed")
+	#net<-graph_from_adjacency_matrix(adjmatrix = matr,weighted = TRUE,mode="directed")
 	return(net)
 }
 
@@ -316,6 +326,8 @@ setMethod("plot", signature(x="markovchain", y="missing"),
 )
 
 
+
+
 #@TAE: create an internal function that does this. Check also if the canonic form function 
 #is appropriate
 
@@ -671,30 +683,41 @@ setMethod("*", c("markovchain","numeric"),
 setMethod("==", c("markovchain","markovchain"),
           function(e1, e2) {
             out <- FALSE
-            out <- identical(e1@transitionMatrix, e2@transitionMatrix)
+            out <-
+              identical(e1@transitionMatrix, e2@transitionMatrix)
             return(out)
-          }
-)
+          })
 
+setMethod("!=", c("markovchain","markovchain"),
+          function(e1, e2) {
+            out <- FALSE
+            out <-
+              (!(identical(
+                e1@transitionMatrix, e2@transitionMatrix
+              )))
+            return(out)
+          })
 
 setMethod("^", c("markovchain", "numeric"),
-function(e1, e2) {
-	 out <- new("markovchain", states=e1@states, byrow=e1@byrow,transitionMatrix=e1@transitionMatrix%^%e2,
-			 name=paste(e1@name,"^",e2,sep=""))
-	 return(out)
-}
-)
+          function(e1, e2) {
+            out <-
+              new(
+                "markovchain", states = e1@states, byrow = e1@byrow,transitionMatrix = e1@transitionMatrix %^%
+                  e2,
+                name = paste(e1@name,"^",e2,sep = "")
+              )
+            return(out)
+          })
 
 
 #methods to directly access transition matrix elements
 
 setMethod("[",
-		signature(x = "markovchain", i = "ANY", j = "ANY"),
-		function(x, i, j) {
-			out <- x@transitionMatrix[i,j]
-			return(out)
-		}
-)
+          signature(x = "markovchain", i = "ANY", j = "ANY"),
+          function(x, i, j) {
+            out <- x@transitionMatrix[i,j]
+            return(out)
+          })
 
 #methods to directly access markovchain objects composing a markovchainList object
 
